@@ -4,13 +4,17 @@ from datetime import date
 from datetime import datetime
 from datetime import time
 from datetime import timedelta
+from typing import Callable
 
 import pytest
 import pytz
+from click.testing import CliRunner
 from freezegun import freeze_time
 
 from tests.helpers import pyicu_sensitive
 from todoman.cli import cli
+from todoman.formatters import DefaultFormatter
+from todoman.formatters import HumanizedFormatter
 from todoman.formatters import rgb_to_ansi
 
 
@@ -18,7 +22,13 @@ from todoman.formatters import rgb_to_ansi
 @pytest.mark.parametrize("interval", [(65, "in a minute"), (-10800, "3 hours ago")])
 @pytest.mark.parametrize("tz", ["CET", "HST"])
 @freeze_time("2017-03-25")
-def test_humanized_datetime(runner, create, interval, now_for_tz, tz):
+def test_humanized_datetime(
+    runner: CliRunner,
+    create: Callable,
+    interval: tuple[int, str],
+    now_for_tz: Callable[[str], datetime],
+    tz: str,
+) -> None:
     seconds, expected = interval
     due = now_for_tz(tz) + timedelta(seconds=seconds)
     create(
@@ -37,7 +47,13 @@ def test_humanized_datetime(runner, create, interval, now_for_tz, tz):
 @pytest.mark.parametrize("interval", [(65, "today"), (-10800, "today")])
 @pytest.mark.parametrize("tz", ["CET", "HST"])
 @freeze_time("2017-03-25 18:00:00")
-def test_humanized_date(runner, create, interval, now_for_tz, tz):
+def test_humanized_date(
+    runner: CliRunner,
+    create: Callable,
+    interval: tuple[int, str],
+    now_for_tz: Callable[[str], datetime],
+    tz: str,
+) -> None:
     seconds, expected = interval
     due = now_for_tz(tz) + timedelta(seconds=seconds)
     create(
@@ -52,7 +68,7 @@ def test_humanized_date(runner, create, interval, now_for_tz, tz):
     assert expected in result.output
 
 
-def test_format_priority(default_formatter):
+def test_format_priority(default_formatter: DefaultFormatter) -> None:
     assert default_formatter.format_priority(None) == "none"
     assert default_formatter.format_priority(0) == "none"
     assert default_formatter.format_priority(5) == "medium"
@@ -65,7 +81,7 @@ def test_format_priority(default_formatter):
         assert default_formatter.format_priority(12)
 
 
-def test_format_priority_compact(default_formatter):
+def test_format_priority_compact(default_formatter: DefaultFormatter) -> None:
     assert default_formatter.format_priority_compact(None) == ""
     assert default_formatter.format_priority_compact(0) == ""
     assert default_formatter.format_priority_compact(5) == "!!"
@@ -78,18 +94,18 @@ def test_format_priority_compact(default_formatter):
         assert default_formatter.format_priority_compact(12)
 
 
-def test_format_date(default_formatter):
+def test_format_date(default_formatter: DefaultFormatter) -> None:
     assert default_formatter.format_datetime(date(2017, 3, 4)) == "2017-03-04"
 
 
-def test_format_datetime(default_formatter):
+def test_format_datetime(default_formatter: DefaultFormatter) -> None:
     assert (
         default_formatter.format_datetime(datetime(2017, 3, 4, 17, 00))
         == "2017-03-04 17:00"
     )
 
 
-def test_detailed_format(runner, todo_factory):
+def test_detailed_format(runner: CliRunner, todo_factory: Callable) -> None:
     todo_factory(
         description=(
             "Test detailed formatting\nThis includes multiline descriptions\nBlah!"
@@ -114,7 +130,7 @@ def test_detailed_format(runner, todo_factory):
     assert result.output.strip().splitlines() == expected
 
 
-def test_parse_time(default_formatter):
+def test_parse_time(default_formatter: DefaultFormatter) -> None:
     tz = pytz.timezone("CET")
     parsed = default_formatter.parse_datetime("12:00")
     expected = datetime.combine(
@@ -124,7 +140,7 @@ def test_parse_time(default_formatter):
     assert parsed == expected
 
 
-def test_parse_datetime(default_formatter):
+def test_parse_datetime(default_formatter: DefaultFormatter) -> None:
     tz = pytz.timezone("CET")
 
     parsed = default_formatter.parse_datetime("2017-03-05")
@@ -144,7 +160,7 @@ def test_parse_datetime(default_formatter):
     assert default_formatter.parse_datetime(None) is None
 
 
-def test_humanized_parse_datetime(humanized_formatter):
+def test_humanized_parse_datetime(humanized_formatter: HumanizedFormatter) -> None:
     tz = pytz.timezone("CET")
 
     humanized_formatter.now = datetime(2017, 3, 6, 22, 17).replace(tzinfo=tz)
@@ -154,20 +170,25 @@ def test_humanized_parse_datetime(humanized_formatter):
     assert humanized_formatter.format_datetime(None) == ""
 
 
-def test_simple_action(default_formatter, todo_factory):
+def test_simple_action(
+    default_formatter: DefaultFormatter,
+    todo_factory: Callable,
+) -> None:
     todo = todo_factory()
     assert default_formatter.simple_action("Delete", todo) == 'Delete "YARR!"'
 
 
-def test_formatting_parsing_consitency(default_formatter):
+def test_formatting_parsing_consitency(default_formatter: DefaultFormatter) -> None:
     tz = pytz.timezone("CET")
     dt = datetime(2017, 3, 8, 21, 6).replace(tzinfo=tz)
 
     formatted = default_formatter.format_datetime(dt)
+    assert formatted is not None
+    assert isinstance(formatted, str)  # FIXME: see default_formatter.format_datetime
     assert default_formatter.parse_datetime(formatted) == dt
 
 
-def test_rgb_to_ansi():
+def test_rgb_to_ansi() -> None:
     assert rgb_to_ansi(None) is None
     assert rgb_to_ansi("#8ab6d") is None
     assert rgb_to_ansi("#8ab6d2f") == "\x1b[38;2;138;182;210m"
@@ -175,7 +196,10 @@ def test_rgb_to_ansi():
     assert rgb_to_ansi("#8ab6d2") == "\x1b[38;2;138;182;210m"
 
 
-def test_format_multiple_with_list(default_formatter, todo_factory):
+def test_format_multiple_with_list(
+    default_formatter: DefaultFormatter,
+    todo_factory: Callable,
+) -> None:
     todo = todo_factory()
     assert todo.list
     assert (
@@ -184,7 +208,10 @@ def test_format_multiple_with_list(default_formatter, todo_factory):
     )
 
 
-def test_format_multiple_without_list(default_formatter, todo_factory):
+def test_format_multiple_without_list(
+    default_formatter: DefaultFormatter,
+    todo_factory: Callable,
+) -> None:
     todo = todo_factory()
     todo.list = None
     assert not todo.list
