@@ -1,31 +1,45 @@
 from __future__ import annotations
 
 from datetime import datetime
+from typing import Callable
 from unittest import mock
 
+import py
 import pytest
 import pytz
 from freezegun import freeze_time
 from urwid import ExitMainLoop
 
+from todoman.formatters import DefaultFormatter
+from todoman.formatters import Formatter
 from todoman.interactive import TodoEditor
+from todoman.model import Database
 
 
-def test_todo_editor_priority(default_database, todo_factory, default_formatter):
+def test_todo_editor_priority(
+    default_database: Database,
+    todo_factory: Callable,
+    default_formatter: DefaultFormatter,
+) -> None:
     todo = todo_factory(priority=1)
     lists = list(default_database.lists())
 
     editor = TodoEditor(todo, lists, default_formatter)
     assert editor._priority.label == "high"
 
-    editor._priority.keypress(10, "right")
+    editor._priority.keypress((10,), "right")
     with pytest.raises(ExitMainLoop):  # Look at editor._msg_text if this fails
         editor._keypress("ctrl s")
 
     assert todo.priority == 0
 
 
-def test_todo_editor_list(default_database, todo_factory, default_formatter, tmpdir):
+def test_todo_editor_list(
+    default_database: Database,
+    todo_factory: Callable,
+    default_formatter: Formatter,
+    tmpdir: py.path.local,
+) -> None:
     tmpdir.mkdir("another_list")
 
     default_database.paths = [
@@ -53,7 +67,11 @@ def test_todo_editor_list(default_database, todo_factory, default_formatter, tmp
     assert another_list.label == todo.list.name
 
 
-def test_todo_editor_summary(default_database, todo_factory, default_formatter):
+def test_todo_editor_summary(
+    default_database: Database,
+    todo_factory: Callable,
+    default_formatter: Formatter,
+) -> None:
     todo = todo_factory()
     lists = list(default_database.lists())
 
@@ -68,7 +86,11 @@ def test_todo_editor_summary(default_database, todo_factory, default_formatter):
 
 
 @freeze_time("2017-03-04 14:00:00", tz_offset=4)
-def test_todo_editor_due(default_database, todo_factory, default_formatter):
+def test_todo_editor_due(
+    default_database: Database,
+    todo_factory: Callable,
+    default_formatter: DefaultFormatter,
+) -> None:
     tz = pytz.timezone("CET")
 
     todo = todo_factory(due=datetime(2017, 3, 4, 14))
@@ -85,18 +107,23 @@ def test_todo_editor_due(default_database, todo_factory, default_formatter):
     assert todo.due == datetime(2017, 3, 10, 12, tzinfo=tz)
 
 
-def test_toggle_help(default_database, default_formatter, todo_factory):
+def test_toggle_help(
+    default_database: Database,
+    default_formatter: Formatter,
+    todo_factory: Callable,
+) -> None:
     todo = todo_factory()
     lists = list(default_database.lists())
 
     editor = TodoEditor(todo, lists, default_formatter)
-    editor._loop = mock.MagicMock()
+    editor._loop = mock.MagicMock()  # type: ignore[assignment]
     assert editor._help_text not in editor.left_column.body.contents
 
     editor._keypress("f1")
     # Help text is made visible
     assert editor._help_text in editor.left_column.body.contents
 
+    assert editor._loop
     # Called event_loop.draw_screen
     assert editor._loop.draw_screen.call_count == 1
     assert editor._loop.draw_screen.call_args == mock.call()
@@ -110,7 +137,11 @@ def test_toggle_help(default_database, default_formatter, todo_factory):
     assert editor._loop.draw_screen.call_args == mock.call()
 
 
-def test_show_save_errors(default_database, default_formatter, todo_factory):
+def test_show_save_errors(
+    default_database: Database,
+    default_formatter: Formatter,
+    todo_factory: Callable,
+) -> None:
     todo = todo_factory()
     lists = list(default_database.lists())
 
@@ -128,7 +159,12 @@ def test_show_save_errors(default_database, default_formatter, todo_factory):
 
 @pytest.mark.parametrize("completed", [True, False])
 @pytest.mark.parametrize("check", [True, False])
-def test_save_completed(check, completed, default_formatter, todo_factory):
+def test_save_completed(
+    check: bool,
+    completed: bool,
+    default_formatter: Formatter,
+    todo_factory: Callable,
+) -> None:
     todo = todo_factory()
     if completed:
         todo.complete()
@@ -141,7 +177,7 @@ def test_save_completed(check, completed, default_formatter, todo_factory):
 
 
 @pytest.mark.skip("See: https://github.com/pimutils/todoman/issues/537")
-def test_ctrl_c_clears(default_formatter, todo_factory):
+def test_ctrl_c_clears(default_formatter: Formatter, todo_factory: Callable) -> None:
     todo = todo_factory()
     editor = TodoEditor(todo, [todo.list], default_formatter)
 
