@@ -1,38 +1,42 @@
 from __future__ import annotations
 
+from abc import ABC
 
-class TodomanError(Exception):
+
+class TodomanError(Exception, ABC):
     """
     Base class for all our exceptions.
 
     Should not be raised directly.
     """
 
+    EXIT_CODE: int
+
 
 class NoSuchTodoError(TodomanError):
     EXIT_CODE = 20
 
-    def __str__(self):
+    def __str__(self) -> str:
         return f"No todo with id {self.args[0]}."
 
 
 class ReadOnlyTodoError(TodomanError):
     EXIT_CODE = 21
 
-    def __str__(self):
+    def __str__(self) -> str:
         return (
-            "Todo is in read-only mode because there are multiple todosin {}.".format(
-                self.args[0]
-            )
+            "Todo is in read-only mode because there are multiple todos "
+            f"in {self.args[0]}."
         )
 
 
 class NoListsFoundError(TodomanError):
     EXIT_CODE = 22
 
-    def __str__(self):
-        return "No lists found matching {}, create a directory for a new list.".format(
-            self.args[0]
+    def __str__(self) -> str:
+        return (
+            f"No lists found matching {self.args[0]}, create a "
+            "directory for a new list."
         )
 
 
@@ -46,5 +50,5 @@ class AlreadyExistsError(TodomanError):
 
     EXIT_CODE = 23
 
-    def __str__(self):
+    def __str__(self) -> str:
         return "More than one {} has the same identity: {}.".format(*self.args)
